/***************************************
	Auteur : Pierre Aubert
	Mail : pierre.aubert@lapp.in2p3.fr
	Licence : CeCILL-C
****************************************/

#ifndef __PGENERIC_SOCKET_MANAGER_H_IMPL__
#define __PGENERIC_SOCKET_MANAGER_H_IMPL__

#include "PGenericSocketManager.h"

///Default constructor of PGenericSocketManager
/**	@param mode : Mode of the Socket (no mock, mock, mock_record)
*/
template<typename _TSocketKey, typename _TBackend, typename _TMockBackend>
PGenericSocketManager<_TSocketKey, _TBackend, _TMockBackend>::PGenericSocketManager(PSocketMode::PSocketMode mode){
	initialisationPGenericSocketManager(mode);
}

///Destructor of PGenericSocketManager
template<typename _TSocketKey, typename _TBackend, typename _TMockBackend>
PGenericSocketManager<_TSocketKey, _TBackend, _TMockBackend>::~PGenericSocketManager(){
	clear();
}

///Set if the current PGenericSocketManager is a mock
/**	@param mode : Mode of the Socket (no mock, mock, mock_record)
*/
template<typename _TSocketKey, typename _TBackend, typename _TMockBackend>
void PGenericSocketManager<_TSocketKey, _TBackend, _TMockBackend>::setMode(PSocketMode::PSocketMode mode){
	p_mode = mode;
}

///Get if the current PGenericSocketManager is a mock
/**	@return Mode of the Socket (no mock, mock, mock_record)
*/
template<typename _TSocketKey, typename _TBackend, typename _TMockBackend>
PSocketMode::PSocketMode PGenericSocketManager<_TSocketKey, _TBackend, _TMockBackend>::getMode() const{
	return p_mode;
}

///Create a client socket
/**	@param name : name (key) to get the socket
 * 	@param param : extra customisable parameters for the creation of the socket (depends on the backend)
 * 	@param mockParam : parameters to initialise the mock
 * 	@return true if the socket has been created, false otherwise
*/
template<typename _TSocketKey, typename _TBackend, typename _TMockBackend>
bool PGenericSocketManager<_TSocketKey, _TBackend, _TMockBackend>::addClientSocket(const _TSocketKey & name, const typename _TBackend::Param & param, const typename _TMockBackend::Param & mockParam)
{
	PGenericSocket<_TBackend, _TMockBackend>* socket = new PGenericSocket<_TBackend, _TMockBackend>(p_mode);
	bool b(socket != NULL);
	if(b){
		b &= socket->createClientSocket(param, mockParam);
		if(b){
			p_mapSocket[name] = socket;
		}
	}
	return b;
}

///Create a server socket
/**	@param name : name (key) to get the socket
 * 	@param param : extra customisable parameters for the creation of the socket (depends on the backend)
 * 	@param mockParam : parameters to initialise the mock
 * 	@return true if the socket has been created, false otherwise
*/
template<typename _TSocketKey, typename _TBackend, typename _TMockBackend>
bool PGenericSocketManager<_TSocketKey, _TBackend, _TMockBackend>::addServerSocket(const _TSocketKey & name, const typename _TBackend::Param & param, const typename _TMockBackend::Param & mockParam)
{
	PGenericSocket<_TBackend, _TMockBackend>* socket = new PGenericSocket<_TBackend, _TMockBackend>(p_mode);
	bool b(socket != NULL);
	if(b){
		b &= socket->createServerSocket(param, mockParam);
		if(b){
			p_mapSocket[name] = socket;
		}
	}
	return b;
}

///Remove the given socket
/**	@param name : name (key) of the socket to be removed
*/
template<typename _TSocketKey, typename _TBackend, typename _TMockBackend>
void PGenericSocketManager<_TSocketKey, _TBackend, _TMockBackend>::removeSocket(const _TSocketKey & name){
	typename std::map<_TSocketKey, PGenericSocket<_TBackend, _TMockBackend>* >::iterator it(p_mapSocket.find(name));
	if(it != p_mapSocket.end()){
		delete it->second;
		p_mapSocket.erase(it);
	}
}

///Clear the map of socket
template<typename _TSocketKey, typename _TBackend, typename _TMockBackend>
void PGenericSocketManager<_TSocketKey, _TBackend, _TMockBackend>::clear(){
	for(typename std::map<_TSocketKey, PGenericSocket<_TBackend, _TMockBackend>* >::iterator it(p_mapSocket.begin()); it != p_mapSocket.end(); ++it){
		delete it->second;
	}
	p_mapSocket.clear();
}

///Send message on the given socket
/**	@param name : name of the socket to be used
* 	@param msg : message to be sent
* 	@param flag : flags to be used to send the message (BLOCK, NON_BLOCK, etc)
* 	@return true on success, false otherwise
*/
template<typename _TSocketKey, typename _TBackend, typename _TMockBackend>
bool PGenericSocketManager<_TSocketKey, _TBackend, _TMockBackend>::sendMsg(const _TSocketKey & name, typename _TBackend::Message & msg, PSendFlag::PSendFlag flag){
	PGenericSocket<_TBackend, _TMockBackend> * socket = getSocket(name);
	if(socket != NULL){
		return socket->sendMsg(msg, flag);
	}else{
		return false;
	}
}

///Recieve message from the given socket
/**	@param name : name of the socket to be used
* 	@param msg : message to be recieved
* 	@param flag : flags to be used to send the message (BLOCK, NON_BLOCK, etc)
* 	@return true on success, false otherwise
*/
template<typename _TSocketKey, typename _TBackend, typename _TMockBackend>
bool PGenericSocketManager<_TSocketKey, _TBackend, _TMockBackend>::recvMsg(const _TSocketKey & name, typename _TBackend::Message & msg, PRecvFlag::PRecvFlag flag){
	PGenericSocket<_TBackend, _TMockBackend> * socket = getSocket(name);
	if(socket != NULL){
		return socket->recvMsg(msg, flag);
	}else{
		return false;
	}
}

///Get a socket by name (or key)
/**	@param name : of the socket to be used
 * 	@return pointer to the found socket, or NULL if the socket does not exist
*/
template<typename _TSocketKey, typename _TBackend, typename _TMockBackend>
PGenericSocket<_TBackend, _TMockBackend>* PGenericSocketManager<_TSocketKey, _TBackend, _TMockBackend>::getSocket(const _TSocketKey & name)
{
	typename std::map<_TSocketKey, PGenericSocket<_TBackend, _TMockBackend>* >::iterator it(p_mapSocket.find(name));
	if(it != p_mapSocket.end()){
		return it->second;
	}else{
		return NULL;
	}
}

///Say if the socket exist with the given name
/**	@param name : of the socket to be used
 * 	@return true if the socket exists, false otherwise
*/
template<typename _TSocketKey, typename _TBackend, typename _TMockBackend>
bool PGenericSocketManager<_TSocketKey, _TBackend, _TMockBackend>::isSocketExist(const _TSocketKey & name) const{
	typename std::map<_TSocketKey, PGenericSocket<_TBackend, _TMockBackend>* >::const_iterator it(p_mapSocket.find(name));
	return it != p_mapSocket.end();
}

///Say if the given socket is connected
/**	@param name : name of the socket to be checked
 * 	@return true if the socket exists and is connected, false otherwise
*/
template<typename _TSocketKey, typename _TBackend, typename _TMockBackend>
bool PGenericSocketManager<_TSocketKey, _TBackend, _TMockBackend>::isConnected(const _TSocketKey & name) const{
	typename std::map<_TSocketKey, PGenericSocket<_TBackend, _TMockBackend>* >::const_iterator it(p_mapSocket.find(name));
	if(it != p_mapSocket.end()){
		return it->second->isConnected();
	}
	return false;
}


///Initialisation function of the class PGenericSocketManager
/**	@param mode : Mode of the Socket (no mock, mock, mock_record)
*/
template<typename _TSocketKey, typename _TBackend, typename _TMockBackend>
void PGenericSocketManager<_TSocketKey, _TBackend, _TMockBackend>::initialisationPGenericSocketManager(PSocketMode::PSocketMode mode){
	p_mode = mode;
}





#endif



