/***************************************
	Auteur : Pierre Aubert
	Mail : pierre.aubert@lapp.in2p3.fr
	Licence : CeCILL-C
****************************************/

#ifndef __PGENERIC_SOCKET_H_IMPL__
#define __PGENERIC_SOCKET_H_IMPL__

#include "PGenericSocket.h"

///Default constructor of PGenericSocket
/**	@param mode : Mode of the Socket (no mock, mock, mock_record)
*/
template<typename _TBackend, typename _TMockBackend>
PGenericSocket<_TBackend, _TMockBackend>::PGenericSocket(PSocketMode::PSocketMode mode){
	initialisationPGenericSocket(mode);
}

///Destructor of PGenericSocket
template<typename _TBackend, typename _TMockBackend>
PGenericSocket<_TBackend, _TMockBackend>::~PGenericSocket(){
	close();
}

///Create a client socket
/**	@param param : extra customisable parameters for the creation of the socket (depends on the backend)
 * 	@param mockParam : parameters of mock backend
 * 	@return true if the socket has been created, false otherwise
*/
template<typename _TBackend, typename _TMockBackend>
bool PGenericSocket<_TBackend, _TMockBackend>::createClientSocket(const typename _TBackend::Param & param, const typename _TMockBackend::Param & mockParam){
	bool b(true);
	if(p_mode != PSocketMode::NO_MOCK){
		b &= _TMockBackend::createClientSocket(p_mockSocket, mockParam);
	}
	if(p_mode != PSocketMode::MOCK){
		b &= _TBackend::createClientSocket(p_socket, param);
	}
	return b;
}

///Create a server socket
/**	@param param : extra customisable parameters for the creation of the socket (depends on the backend)
 * 	@param mockParam : parameters of mock backend
 * 	@return true if the socket has been created, false otherwise
*/
template<typename _TBackend, typename _TMockBackend>
bool PGenericSocket<_TBackend, _TMockBackend>::createServerSocket(const typename _TBackend::Param & param, const typename _TMockBackend::Param & mockParam){
	bool b(true);
	if(p_mode != PSocketMode::NO_MOCK){
		b &= _TMockBackend::createServerSocket(p_mockSocket, mockParam);
	}
	if(p_mode != PSocketMode::MOCK){
		b &= _TBackend::createServerSocket(p_socket, param);
	}
	return b;
}

///Send message on the given socket
/**	@param msg : message to be sent
* 	@param flag : flags to be used to send the message (BLOCK, NON_BLOCK, etc)
* 	@return true on success, false otherwise
*/
template<typename _TBackend, typename _TMockBackend>
bool PGenericSocket<_TBackend, _TMockBackend>::sendMsg(typename _TBackend::Message & msg, PSendFlag::PSendFlag flag){
	size_t dataSize(msg.size());
	bool b(true);
	if(p_mode != PSocketMode::NO_MOCK){
		typename _TMockBackend::Message vecTmp;
		_TMockBackend::msgResize(vecTmp, dataSize);
		memcpy(_TMockBackend::msgData(vecTmp), _TBackend::msgData(msg), dataSize);
		b &= _TMockBackend::send(p_mockSocket, vecTmp, flag);
	}
	if(p_mode != PSocketMode::MOCK){
		b &= _TBackend::send(p_socket, msg, flag);
	}
	return b;
}

///Recieve message from the given socket
/**	@param msg : message to be recieved
* 	@param flag : flags to be used to send the message (BLOCK, NON_BLOCK, etc)
* 	@return true on success, false otherwise
*/
template<typename _TBackend, typename _TMockBackend>
bool PGenericSocket<_TBackend, _TMockBackend>::recvMsg(typename _TBackend::Message & msg, PRecvFlag::PRecvFlag flag){
	bool b(true);
	if(p_mode == PSocketMode::NO_MOCK){	//Normal mode
		b &= _TBackend::recv(p_socket, msg, flag);
	}else if(p_mode == PSocketMode::MOCK){	//Mock mode
		typename _TMockBackend::Message msgMock;
		b &= _TMockBackend::recv(p_mockSocket, msgMock, flag);
		if(b){
			size_t dataSize(_TMockBackend::msgSize(msgMock));
			_TBackend::msgResize(msg, dataSize);
			memcpy(_TBackend::msgData(msg), _TMockBackend::msgData(msgMock), dataSize);
		}
	}else{					//Mock record mode
		b &= _TBackend::recv(p_socket, msg, flag);
		if(b){
			typename _TMockBackend::Message msgMock;
			size_t dataSize(_TBackend::msgSize(msg));
			_TMockBackend::msgResize(msgMock, dataSize);
			memcpy(_TMockBackend::msgData(msgMock), _TBackend::msgData(msg), dataSize);
			b &= _TMockBackend::recv(p_mockSocket, msgMock, flag);
		}
	}
	return b;
}

///Close the socket
template<typename _TBackend, typename _TMockBackend>
void PGenericSocket<_TBackend, _TMockBackend>::close(){
	if(p_mode == PSocketMode::MOCK){
		_TMockBackend::close(p_mockSocket);
	}else if(p_mode == PSocketMode::NO_MOCK){
		_TBackend::close(p_socket);
	}else{
		_TMockBackend::close(p_mockSocket);
		_TBackend::close(p_socket);
	}
}

///Say if the Socket is connected
/**	@return true if the current socket is connected (or both in record mode)
*/
template<typename _TBackend, typename _TMockBackend>
bool PGenericSocket<_TBackend, _TMockBackend>::isConnected() const{
	bool b(true);
	if(p_mode != PSocketMode::NO_MOCK){
		b &= _TMockBackend::isConnected(p_mockSocket);
	}
	if(p_mode != PSocketMode::MOCK){
		b &= _TBackend::isConnected(p_socket);
	}
	return b;
}

///Initialisation function of the class PGenericSocket
/**	@param mode : Mode of the Socket (no mock, mock, mock_record)
*/
template<typename _TBackend, typename _TMockBackend>
void PGenericSocket<_TBackend, _TMockBackend>::initialisationPGenericSocket(PSocketMode::PSocketMode mode){
	p_mode = mode;
}


#endif



